import unittest
from mock import patch


class TestConversion(unittest.TestCase):

    def test_convert(self):
        """Test :py:func:``exchange.conversion.convert``"""
        from exchange.conversion import convert, Price
        with patch('exchange.conversion.ExchangeRates') as exchange_rates:
            exchange_rates.get_instance.return_value = \
                {'USD': {'GBP': 0.5}}
            price = Price(3, 'USD')
            converted_price = convert(price, 'GBP')
            self.assertEqual(converted_price.value, 1.50)
            self.assertEqual(converted_price.currency, 'GBP')

    def test_price(self):
        """Test :py:class:``exchange.conversion.Price``"""
        from exchange.conversion import Price
        with patch('exchange.conversion.ExchangeRates') as exchange_rates:
            exchange_rates.get_instance.return_value = \
                {'USD': {'GBP': 0.5}}
            price = Price(3, 'USD')
            converted_price = price.convert('GBP')
            self.assertEqual(converted_price.value, 1.50)
            self.assertEqual(converted_price.currency, 'GBP')

    def test_exchangerates(self):
        """Test :py:class:``exchange.conversion.ExchangeRates``"""
        from exchange.conversion import ExchangeRates
        from exchange.models import Currency, ExchangeRate
        usd = Currency.objects.create(code='USD')
        gbp = Currency.objects.create(code='GBP')
        afn = Currency.objects.create(code='AFN')

        ExchangeRate.objects.create(source=usd, target=gbp, rate='2.00')
        ExchangeRate.objects.create(source=usd, target=afn, rate='3.00')

        rates = ExchangeRates.get_instance()
        rates.populate()
        self.assertIn('USD', rates)
        self.assertIn('GBP', rates['USD'])
        self.assertEqual(rates['USD']['GBP'], 2.00)
        self.assertIn('AFN', rates['USD'])
        self.assertEqual(rates['USD']['AFN'], 3.00)
        rates2 = ExchangeRates.get_instance()
        self.assertEqual(rates, rates2)
        rates.clear()
        self.assertEqual(len(rates), 0)



from exchange.models import Currency


def import_class(class_path):
    """imports and returns given class string.

    :param class_path: Class path as string
    :type class_path: str

    :returns: Class that has given path
    :rtype: class

    :Example:

    >>> import_class('collections.OrderedDict').__name__
    'OrderedDict'
    """
    try:
        from django.utils.importlib import import_module
        module_name = '.'.join(class_path.split(".")[:-1])
        mod = import_module(module_name)
        return getattr(mod, class_path.split(".")[-1])
    except Exception, detail:
        raise ImportError(detail)



from django.db import models


class Currency(models.Model):
    """Model holds a currency information for a nationality"""
    code = models.CharField(max_length=3, unique=True)
    name = models.CharField(max_length=64)

    def __unicode__(self):
        return self.code


class ExchangeRate(models.Model):
    """Model to persist exchange rates between currencies"""
    source = models.ForeignKey('exchange.Currency', related_name='rates')
    target = models.ForeignKey('exchange.Currency')
    rate = models.DecimalField(max_digits=12, decimal_places=2)

    def __unicode__(self):
        return '%s / %s = %s' % (self.source, self.target, self.rate)



from exchange.models import Currency, ExchangeRate


def convert(price, currency):
    """Converts the price of a currency to another one using exhange rates

    :param price: the price value
    :param type: decimal

    :param currency: ISO-4217 currency code
    :param type: str

    :returns: converted price instance
    :rtype: ``Price``

    """
    rates = ExchangeRates.get_instance()
    return Price(price.value * rates[price.currency][currency], currency)


class Price(object):
    """Class holds the information of a price value with its currency"""

    def __init__(self, value, currency):
        """Convenient constrcutor

        :param value: the price value
        :param type: decimal

        :param currency: ISO-4217 currency code
        :param type: str

        """
        self.value = value
        self.currency = currency

    def convert(self, currency):
        """Converts the price of a currency hold by currenct instance to
        another one

        :param currency: ISO-4217 currency code
        :param type: str

        :returns: converted price instance
        :rtype: ``Price``

        """
        return convert(self, currency)

    def __repr__(self):
        return '<Price (%s %s)>' % (self.value, self.currency)


class ExchangeRates(dict):
    """Singleton dictionary implementation which hold the exchange rates
    populated from corresponding database models.

    """
    _instance = None

    @classmethod
    def get_instance(cls):
        """Singleton instance method"""
        if not cls._instance:
            cls._instance = ExchangeRates()
            cls._instance.populate()
        return cls._instance

    def reset(self):
        """Clears all the exchange rate data"""
        self.clear()

    def populate(self):
        """Clears and populates all the exchange rate data via database"""
        self.reset()
        currencies = Currency.objects.all()
        for source_currency in currencies:
            self[source_currency.code] = {}
            for rate in source_currency.rates.all():
                self[source_currency.code][rate.target.code] = rate.rate
